#include "cJSON.h"
#include "blunoAccessory.h"
#include "communication.h"
#include "U8glib.h"
#include "Wire.h"
#include "I2Cdev.h"
#include "MPU6050.h"

#define RIGHT 1
#define UP 2
#define LEFT 0
#define DOWN 4
#define PUSH 5
#define MID 3

blunoAccessory myAccessory;

// class default I2C address is 0x68
// specific I2C addresses may be passed as a parameter here
// AD0 low = 0x68 (default for InvenSense evaluation board)
// AD0 high = 0x69
MPU6050 accelgyro;
int ax, ay, az;
int gx, gy, gz;

struct information info;

char *oledStr = "****";

//OLED constructor, for oled display
U8GLIB_SSD1306_128X64 myOled(U8G_I2C_OPT_NONE);

void setup()
{
	Serial.begin(115200);          //  setup serial
	myAccessory.begin();
	
	// initialize device
    accelgyro.initialize();
	// connect MPU6050
	accelgyro.testConnection();
	pinMode(13, OUTPUT);
}


void loop()
{
	unsigned int datLen;
	const char *data = getData(&datLen);

	if(data != NULL){
		cJSON *root = cJSON_Parse(data);
		int type = cJSON_GetObjectItem(root, "T")->valueint;
		int value = cJSON_GetObjectItem(root, "V")->valueint;
		switch(type){
			case BUZZER:{
				myAccessory.setBuzzer(value?true:false);
				}break;
			case RELAY:{
				myAccessory.setRelay(value?true:false);
				}break;
			case RGBLED:{
				int red = 255-cJSON_GetObjectItem(root, "R")->valueint;
				int green = 255-cJSON_GetObjectItem(root, "G")->valueint;
				int blue = 255-cJSON_GetObjectItem(root, "B")->valueint;
				myAccessory.setRGBLed(red, green, blue);
			}break;
			case TEMPERATURE:{
				float t = myAccessory.readTemperature();
				sendTemperature(t);
			}break;
			case HUMIDITY:{
				float h = myAccessory.readHumidity();
				sendHumidity(h);
			}break;
			case OLED:{
				oledStr = cJSON_GetObjectItem(root, "S")->valuestring;
				myOled.firstPage();                 //for more information about the U8glib library, go to https://code.google.com/p/u8glib/
		        do{
		            myOled.setFont(u8g_font_unifont);
					myOled.setPrintPos(10,16);      //set the print position
				    myOled.print(oledStr);
		        }
		        while(myOled.nextPage());
			}break;
			case DIGITAL:
			case ANALOG:{
				int pin = cJSON_GetObjectItem(root, "P")->valueint;
				int mode = cJSON_GetObjectItem(root, "M")->valueint;
				if(mode == WRITE){
					pinMode(pin, OUTPUT);
					if(type == DIGITAL){
						digitalWrite(pin, value?HIGH:LOW);
					}else{
						analogWrite(pin, value);
					}
				}else{
					pinMode(pin, INPUT);
					int value;
					if(type == DIGITAL){
						value = digitalRead(pin);
					}else{
						value = analogRead(pin);
					}
					char jsonBuf[40];
					sprintf(jsonBuf, "{\"T\":%d, \"V\":%d, \"P\":%d}", type, value, pin);
					sendData(jsonBuf);
				}
			}break;
			default:break;
		}
		cJSON_Delete(root);
	}

	static unsigned long sendInfoTimer = millis();        //every 300ms update the information
    if (millis() - sendInfoTimer >= 300) {
        sendInfoTimer = millis();
        sendInformation(&info);
    }

	static unsigned long DHT11Timer = millis();       //every 2s update the temperature and humidity from DHT11 sensor
    if (millis() - DHT11Timer >= 2000) {
        DHT11Timer = millis();
        info.temperature = myAccessory.readTemperature();
        info.humidity = myAccessory.readHumidity();
    }
    
    static unsigned long Timer300ms = millis();       //every 300ms update the knob(potentiometer) and the joystick and mpu6050
    if (millis() - Timer300ms >= 300) {
        Timer300ms = millis();
        info.knob = myAccessory.readJoystick_x();
        info.joyStick = myAccessory.readJoystick_y();
		// read raw accel/gyro measurements from device
    	accelgyro.getMotion6(&(info.ax), &(info.ay), &(info.az), &(info.gy), &(info.gy), &(info.gy));
    }
    
    if (myAccessory.joystickAvailable()) {          //if the state of joystick is changed
        info.joyStick = myAccessory.readJoystick();        //update the joystick
    }
}

